@@ -5,31 +5,37 @@
 import os
 import time
 import cv2
+import csv
+import random
 import pyautogui
 import keyboard
 import numpy as np
-import matplotlib.pyplot as plt
 import pdfplumber
+from string import ascii_letters,digits
+#initialize version 0.0.8
+
+# will work and save data to the location of the package / module
 
-"""will work and save data to the location of the package / module"""
 package_path = "\\".join(os.path.abspath(__file__).split("\\")[:-1])+"\\"
 
-def scroll_line(nlines=1):
-    """will use pyautogui to scroll lines - for 150% PDF"""
-    for i in range(nlines):
-        for i in range(3):
-            pyautogui.scroll(-14)
-
-def scroll_page(npages=1):
-    """will us pyautogui to scroll the bit at the end of a page"""
-    for i in range(npages):
-        for i in range(19):
-            pyautogui.scroll(-25)
-
-"""
-statistical functions, 
-utilizing numpy to transform arrays
-"""
+def __help__():
+    """help for scrollthescroll"""
+    print("\n",
+          "|--------------------------------","\n","| scrollthescroll.Prototype","\n","|","\n",
+          "|    Prototype Functions","\n","|","\n",
+          "|    Prototype().run(extract, page_number, MIN_LINES=2, save_dir='data',file_path=package_path)","\n","|","\n",
+          "|     // run the scrollthescroll program //","\n","|","\n",
+          "|     extract - path to PDF file","\n","|","\n",
+          "|     page_number - page number to begin reading from","\n","|","\n",
+          "|     MIN_LINES=2 - minimum 'read' lines before scroll","\n","|","\n",
+          "|     save_dir='data' - location for data to be saved","\n","|","\n",
+          "|     file_path=package_path - path to package","\n","|","\n",
+          "|    Prototype().run(file_path=package)","\n","|","\n",
+          "|     // test the program will function //","\n","|","\n",
+          "|     iterations=1","\n","|","\n",
+          "|     file_path=package_path - path to package","\n","|","\n",
+          "|--------------------------------","\n","|","\n"
+          )
 
 def minmax(array):
     """(x-min)/(max-min)"""
@@ -60,63 +66,13 @@ def format_package(package,STANDARD_DEVIATIONS=3):
     packets = list(map(minmax,packets))
     return [packet_times,packets]
 
-"""
-independent packet functions,
-functions for saving and loading any data
-functions are not used within the 
-"""
-def create_data_directory(save_dir="packageSamples",file_path=package_path):
+def create_data_directory(save_dir="data",file_path=package_path):
     """creates directory for storing past samples"""
-    datapath = os.path.join("data","")
+    datapath = os.path.join(file_path,save_dir)
     try:
-        next(os.walk(file_path+datapath))
+        next(os.walk(datapath))
     except StopIteration:
-        os.mkdir(file_path+datapath)
-    datapath = os.path.join("data",save_dir,"")
-    try:
-        next(os.walk(file_path+datapath))
-    except StopIteration:
-        os.mkdir(file_path+datapath)
-
-def load_extract_index(save_dir="packageSamples",file_path=package_path):
-    """index for saved samples"""
-    extract_index = {}
-    def assign_key(elem):
-        # for assinging keys for a dictionary
-        key,content = elem.split(":")
-        extract,page_number = content.split(",")
-        extract_index[key] = (extract,int(page_number))
-    datapath = os.path.join("data",save_dir,"")
-    with open(file_path+datapath+"extract_index.txt","r") as file:
-        list(map(assign_key,file.read().split("|")[1:]))
-    return extract_index
-
-def load_data(directory_index,package_index,save_dir="packageSamples",file_path=package_path):
-    """loading a saved sample in form [[times],[packets]]"""
-    #getting the location of the desired data
-    datapath = os.path.join("data",save_dir,"")
-    _,directories,_ = next(os.walk(file_path+datapath))
-    directory = sorted(directories,key=lambda elem:int(elem[7:]))[directory_index]
-    datapath = os.path.join("data",save_dir,directory,"")
-    _,_,packages = next(os.walk(file_path+datapath))
-    package   = sorted(packages,key= lambda elem:int(elem[8:-4]))[package_index]
-    #loading from .txt file
-    with open(file_path+datapath+package,"r") as file:
-        times,packets = file.read().split("|")
-    #transferring from string to array
-    #packets saved in form [x,y],[x,y],...
-    packets = list(map(lambda elem: [float(i) for i in elem.split(",")],packets.split("-")))
-    times   = list(map(float,times.split("-")))
-    return [times,packets]
-
-"""
-pupil functions,
-pupil functions work with good level of success however the population that they have been 
-tested on is very small and is the most likely part of the prototype to fail - will require 
-larger test population or data-set to perfect. (finding eyes and finding face is reliable for it uses
-the trusted haars method from cv2 - utilizes larger data population - finding pupils is weak part).
-Integral part of run
-"""
+        os.mkdir(datapath)
 
 def pupil_contours(roi_eye):
     """finds the ideal contours using regression with average colour and uBound"""
@@ -224,13 +180,6 @@ class PupilFunctions:
                                (face_x+eye_x+x_mean,face_y+eye_y+int(eye_h*0.25)+y_mean),
                                radius=0, color=(0, 255, 0 ), thickness=3)
         return self.pupils
-"""
-packet model is in infancy stages,
-works well however is perhaps overactive when the user is not reading.
-When reading however model works well
-
-once again suffers from low population size and reading rates are generated from small population
-"""
 
 def clean_package(formatted_package):
     """removes head, tail and outliers"""
@@ -354,11 +303,7 @@ class packet_model:
                 self.reading_amount -= minimum_reading_time
             return True
         return False
-"""
-for simply loading PDF into useable pythonic form
-uses the slower pdfplumber as opposed to pdftotext because it is easier to install,
-with pdftotext requiring Anaconda or c++ tools - not a given
-"""
+
 class ExtractFunctions:
     """loading PDF file into pythonic form"""
     def __init__(self):
@@ -396,12 +341,6 @@ class ExtractFunctions:
         self.reading_std = self.reading_rates.std()
         return self.reading_rates
 
-"""
-simple regression functions using numpy
-will calculate gradient,y-intercept and a regression corralation coefficient 
-recycles certain variables for efficiency as functions could be called and used often
-"""
-
 class RegressionFunctions:
     """linear regression functions"""
     def __init__(self):
@@ -424,22 +363,12 @@ class RegressionFunctions:
     def Predict(self,X):
         """y = mX+c"""
         return X*self.gradient+self.y_intercept
-"""
-prototype.
-Will require user to supply and source their own PDF file
-will save the packages by default for analysis however will only involve the dry run tool 
-other analysis tools aren't built to a great degree of userability, need to be upgraded - 
-potentially to be added to later versions
-"""
 
-def display_package(package,background_data):
-    """will show the package against the background_data"""
-    _,ax = plt.subplots(nrows=1)
-    packet_times,packets = background_data
-    ax.scatter(packet_times,list(zip(*packets))[0])
-    packet_times, packets = package
-    ax.scatter(packet_times,list(zip(*packets))[0])
-    plt.pause(0.1)
+def scroll_line(nlines=1):
+    """will use pyautogui to scroll lines - for 150% PDF"""
+    for i in range(nlines):
+        for i in range(3):
+            pyautogui.scroll(-14)
 
 class Prototype:
     """running the current prototype"""
@@ -450,47 +379,18 @@ class Prototype:
 
         self.directory = None
 
-    def dry_run(self,save_dir="packageSamples",file_path=package_path):
-        """will dry run through saved data to ensure model is working"""
-        extract_index = load_extract_index(save_dir=save_dir,file_path=file_path)
-        datapath = os.path.join("data",save_dir,"")
-        _,directories,_ = next(os.walk(datapath))
-        directories = sorted(directories,key=lambda elem:int(elem[7:]))
-        for directory_number,directory in enumerate(directories):
-            extract,page_number = extract_index[directory]
-            self.left_packet_model.load_extract(extract,page_number)
-            self.right_packet_model.load_extract(extract,page_number)
-            left_data = load_data(directory_number,0,
-                                  save_dir=save_dir,file_path=file_path)
-            left_data_iter = list(zip(*left_data))
-            right_data = load_data(directory_number,1,
-                                   save_dir=save_dir,file_path=file_path)
-            right_data_iter = list(zip(*right_data))
-            for index_number in range(len(sorted([left_data_iter,right_data_iter],
-                                     key=len)[0])):
-                packet_time,packet = left_data_iter[index_number]
-                left_package = self.left_packet_model.build_package(packet_time,packet)
-                if left_package:
-                    pass
-                packet_time,packet = right_data_iter[index_number]
-                right_package = self.right_packet_model.build_package(packet_time,packet)
-                if right_package:
-                    pass
-
     def run(self,extract,page_number,MIN_LINES=2,
-            save_dir="packageSamples",file_path=package_path):
-        """running the prototype - for 150% zoom:
-        parameters: MIN_LINES=minimum amount of lines read before scrolling,
-        save_dir=directory to save the sample to, file_path = where directory should be found,
-        call=True/False whether to call PDF doc to PDF reader"""
-        #load extracts for both models
-        extract=extract.split(":")[-1]
-        self.left_packet_model.load_extract(extract,page_number)
+                save_dir="data",file_path=package_path):
+        """extract: path to PDF file, page_number: PDF page to begin reading from,
+        MIN_LINES:minimum amount of lines read before scrolling, save_dir: directory to
+        save sample to,file_path: where directory should be found.
+        """
+        extract = extract.split(":")[-1]
         self.right_packet_model.load_extract(extract,page_number)
-        line_count = 0
+        self.left_packet_model.load_extract(extract,page_number)
+        line_count =0
         start_time = time.perf_counter()
-        complete_left = [[],[]]
-        complete_right= [[],[]]
+        save_data = [[],[],[],[],[]]
         print("preparing webcam, please wait ... ")
         cap = cv2.VideoCapture(0)
         print("ready, <press space>")
@@ -501,8 +401,6 @@ class Prototype:
             _,frame = cap.read()
             left_pupil,right_pupil = self.pupil_functions.find_pupils(frame)
             if left_pupil:
-                complete_left[0].append(time.perf_counter()-start_time)
-                complete_left[1].append(left_pupil)
                 left_package = self.left_packet_model.build_package(time.perf_counter()-start_time,
                                                                     left_pupil)
                 if left_package:
@@ -516,7 +414,6 @@ class Prototype:
                         scroll_line(nlines=MIN_LINES)
                         line_count = 0
                     if self.left_packet_model.extract_functions.page_index != self.right_packet_model.extract_functions.page_index:
-                        scroll_page()
                         line_count = 0
                         self.left_packet_model.reading_amount = 0
                         self.right_packet_model.reading_amount = 0
@@ -525,10 +422,7 @@ class Prototype:
                     self.right_packet_model.package = [[],[]]
                     self.right_packet_model.line_index = self.left_packet_model.line_index
                     self.right_packet_model.reading_amount = self.left_packet_model.reading_amount
-
             if right_pupil:
-                complete_right[0].append(time.perf_counter()-start_time)
-                complete_right[1].append(right_pupil)
                 right_package=self.right_packet_model.build_package(time.perf_counter()-start_time,
                                                                     right_pupil)
                 if right_package:
@@ -541,7 +435,6 @@ class Prototype:
                         scroll_line(nlines=MIN_LINES)
                         line_count = 0
                     if self.right_packet_model.extract_functions.page_index != self.left_packet_model.extract_functions.page_index:
-                        scroll_page()
                         line_count = 0
                         self.right_packet_model.reading_amount = 0
                         self.left_packet_model.reading_amount = 0
@@ -549,6 +442,12 @@ class Prototype:
                     self.left_packet_model.package = [[],[]]
                     self.left_packet_model.line_index = self.right_packet_model.line_index
                     self.left_packet_model.reading_amount = self.right_packet_model.reading_amount
+            if right_pupil and left_pupil:
+                save_data[0].append(time.perf_counter()-start_time)
+                save_data[1].append(left_pupil[0])
+                save_data[2].append(left_pupil[1])
+                save_data[3].append(right_pupil[0])
+                save_data[4].append(right_pupil[1])
             if keyboard.is_pressed("space"):
                 break
             cv2.waitKey(1)
@@ -556,78 +455,20 @@ class Prototype:
         cap.release()
         cv2.destroyAllWindows()
         create_data_directory(save_dir=save_dir,file_path=file_path)
-        self.save_package(complete_left,save_dir=save_dir,file_path=file_path)
-        self.save_package(complete_right,save_dir=save_dir,file_path=file_path)
-        datapath = os.path.join("data",save_dir,"")
-        with open(file_path+datapath+"extract_index.txt","a") as file:
-            file.write("|{}:{},{}".format(self.directory,
-                                          self.left_packet_model.extract_functions.extract,
-                                          page_number))
+        print("saving data, please don't shut down ...")
+        self.save_package(save_data,save_dir=save_dir,file_path=file_path)
+        print("data saved")
 
-    def save_package(self,package,save_dir="packageSamples",file_path=package_path):
-        """saving a prototype run for analysis"""
-        #checking if a directory has been made to save to
-        if not self.directory:
-            datapath = os.path.join("data",save_dir,"")
-            _,directories,_ = next(os.walk(file_path+datapath))
-            try:
-                self.directory = "sample_"+str(max(list(map(lambda elem:int(elem[7:]),
-                                                            directories)))+1)
-            except ValueError:
-                self.directory = "sample_0"
-            datapath = os.path.join("data",save_dir,self.directory)
-            os.mkdir(file_path+datapath)
-        #deciding what the name of the saved package will be
-        datapath = os.path.join("data",save_dir,self.directory,"")
-        _,_,packages = next(os.walk(file_path+datapath))
-        try:
-            package_route = "package_"+str(max(list(map(lambda elem:int(elem[8:-4]),
-                                                       packages)))+1)+".txt"
-        except ValueError:
-            package_route = "package_0.txt"
-        #formatting package to be saved as a .txt file
-        times,packets = package
-        packets = "-".join(list(map(lambda elem:",".join([str(i) for i in elem]),packets)))
-        times   = "-".join([str(i) for i in times])
-        package_string = times+"|"+packets
-        with open(file_path+datapath+package_route,"w") as file:
-            file.write(package_string)
-
-    def display_run(self,lower_bound,upper_bound=None,save_dir="packageSamples",
-                    file_path=package_path):
-        """for visualizing past runs, works best with in-line plotting.
-        Parameters: lower_bound=sample to start from, upper_bound=sample to end on"""
-        extract_index = load_extract_index(save_dir=save_dir,file_path=file_path)
-        datapath = os.path.join("data",save_dir,"")
-        _,samples,_ = next(os.walk(file_path+datapath))
-        if not upper_bound:
-            upper_bound = len(samples)
-        for sample_number,sample in enumerate(samples[lower_bound:upper_bound]):
-            sample_number += lower_bound
-            extract,page_number = extract_index[sample]
-            self.left_packet_model.load_extract(extract,page_number)
-            self.right_packet_model.load_extract(extract,page_number)
-            left_data = load_data(sample_number,0,save_dir=save_dir,file_path=file_path)
-            left_data_iter = list(zip(*left_data))
-            right_data= load_data(sample_number,1,save_dir=save_dir,file_path=file_path)
-            right_data_iter= list(zip(*right_data))
-            for index_number in range(len(sorted([left_data_iter,right_data_iter],
-                                     key=len)[0])):
-                packet_time,packet = left_data_iter[index_number]
-                package = self.left_packet_model.build_package(packet_time,packet)
-                if package:
-                    display_package(package,left_data)
-                    input("press enter")
-                    self.right_packet_model.package = [[],[]]
-                    self.right_packet_model.line_index = self.left_packet_model.line_index
-                    self.right_packet_model.extract_functions.page_index = self.left_packet_model.extract_functions.page_index
-                    self.right_packet_model.reading_amount = self.left_packet_model.reading_amount
-                packet_time,packet = right_data_iter[index_number]
-                package = self.right_packet_model.build_package(packet_time,packet)
-                if package:
-                    display_package(package,right_data)
-                    input("press enter")
-                    self.left_packet_model.package = [[],[]]
-                    self.left_packet_model.line_index = self.right_packet_model.line_index
-                    self.left_packet_model.extract_functions.page_index = self.right_packet_model.extract_functions.page_index
-                    self.left_packet_model.reading_amount = self.right_packet_model.reading_amount
+    def save_package(self,package,save_dir="data",file_path=package_path):
+        """"saving a prototype run for analysis, saves the package to .csv file"""
+        #will need to assign a random hash to the files so that there are no clashes of data in time
+        datapath = os.path.join(file_path,save_dir,"")
+        chars = ascii_letters+digits
+        file_name = "S{}.csv".format("".join([random.choice(chars) for i in range(15)]))
+        datapath = os.path.join(datapath,file_name)
+        with open(datapath,"w",encoding="UTF8") as file:
+            csv_writer = csv.writer(file)
+            header = ["T","Lx","Ly","Rx","Ry"]
+            csv_writer.writerow(header)
+            for row in list(zip(*package)):
+                csv_writer.writerow(row)
